@@ -701,14 +701,33 @@ bool GameArea::IsFocusedCellEditable() const
   return !finished->Get() && riddle.Get(focusedCell)==Sudoku::empty;
 }
 
+bool GameArea::IsCellInMarkModus() const
+{
+  return game.Get(focusedCell)==Sudoku::empty;
+}
+
 size_t GameArea::GetFocusedCellValue() const
 {
-  if (game.Get(focusedCell)!=Sudoku::empty) {
-    return 1 << (game.Get(focusedCell)-1);
-  }
-  else {
-    return marks[focusedCell];
-  }
+  assert(game.Get(focusedCell)!=Sudoku::empty);
+
+  return game.Get(focusedCell);
+}
+
+size_t GameArea::GetFocusedCellMarks() const
+{
+  assert(game.Get(focusedCell)==Sudoku::empty);
+
+  return marks[focusedCell];
+}
+
+void GameArea::SetFocusedCellMarks(size_t value)
+{
+  assert(!GetFinishedModel()->Get());
+  assert(IsFocusedCellEditable());
+
+  game.Set(focusedCell,Sudoku::empty);
+  marks[focusedCell]=value;
+  Redraw();
 }
 
 void GameArea::SetFocusedCellValue(size_t value)
@@ -716,29 +735,10 @@ void GameArea::SetFocusedCellValue(size_t value)
   assert(!GetFinishedModel()->Get());
   assert(IsFocusedCellEditable());
 
-  size_t count=0;
-
-  for (size_t i=0; i<Sudoku::dimension; i++) {
-    if (value & (1 << i)) {
-      count++;
-    }
-  }
-
-  if (count==1) {
-    for (size_t i=0; i<Sudoku::dimension; i++) {
-      if (value & (1 << i)) {
-        game.Set(focusedCell,i+1);
-        marks[focusedCell]=0;
-        finished->Set(game==solution);
-        Redraw();
-      }
-    }
-  }
-  else {
-    game.Set(focusedCell,Sudoku::empty);
-    marks[focusedCell]=value;
-    Redraw();
-  }
+  game.Set(focusedCell,value);
+  marks[focusedCell]=0;
+  finished->Set(game==solution);
+  Redraw();
 }
 
 void GameArea::Solve()
