@@ -29,10 +29,15 @@
 #include <Lum/Dlg/Msg.h>
 #include <Lum/Dlg/Progress.h>
 
+#include <Lum/Images/Image.h>
+#include <Lum/Images/Loader.h>
+
 #include <Lum/Array.h>
+#include <Lum/AspectRatio.h>
 #include <Lum/Button.h>
 #include <Lum/Combo.h>
 #include <Lum/Dialog.h>
+#include <Lum/Image.h>
 #include <Lum/Label.h>
 #include <Lum/Menu.h>
 #include <Lum/Panel.h>
@@ -49,9 +54,10 @@
 #include "Configuration.h"
 #include "Dancing.h"
 #include "GameArea.h"
+#include "GenerateDialog.h"
 #include "Statistics.h"
 
-static Lum::AppInfo info;
+static Lum::Def::AppInfo info;
 
 // TODO
 static Sudoku riddle,solution;
@@ -121,16 +127,20 @@ public:
 class MainDialog : public Lum::Dialog
 {
 private:
-  Lum::Model::ActionRef   generateAction;
-  Lum::Model::ActionRef   solveAction;
-  Lum::Model::ActionRef   statAction;
-  Lum::Model::ActionRef   helpAction;
-  Lum::Model::ActionRef   aboutAction;
-  Lum::Model::StringRef   minimumPoints;
-  GameArea                *gameArea;
-  Lum::Text               *rating;
-  GenerateTask            generateTask;
-  Lum::Model::BooleanRef  keypad[Sudoku::dimension];
+  Lum::Model::ActionRef         generateAction;
+  Lum::Model::ActionRef         solveAction;
+  Lum::Model::ActionRef         statAction;
+  Lum::Model::ActionRef         helpAction;
+  Lum::Model::ActionRef         aboutAction;
+  GameArea                      *gameArea;
+  Lum::Text                     *rating;
+  GenerateTask                  generateTask;
+  Lum::Model::BooleanRef        keypad[Sudoku::dimension];
+  Lum::Model::BooleanRef        mark;
+  bool                          lock;
+  Lum::Images::StatefulImageRef statisticsImage;
+  Lum::Images::StatefulImageRef solveImage;
+  Lum::Images::StatefulImageRef generateImage;
 
 public:
   MainDialog()
@@ -139,19 +149,23 @@ public:
     statAction(new Lum::Model::Action),
     helpAction(new Lum::Model::Action),
     aboutAction(new Lum::Model::Action),
-    minimumPoints(new Lum::Model::String()),
     gameArea(new GameArea()),
-    rating(NULL)
+    rating(NULL),
+    mark(new Lum::Model::Boolean(false)),
+    lock(false),
+    statisticsImage(new Lum::Images::StatefulImage()),
+    solveImage(new Lum::Images::StatefulImage()),
+    generateImage(new Lum::Images::StatefulImage())
   {
     for (size_t i=0; i<Sudoku::dimension; i++) {
       keypad[i]=new Lum::Model::Boolean(false);
       Observe(keypad[i]);
     }
+    Observe(mark);
 
     generateTask.SetParent(this);
     generateTask.SetCaption(_(L"PROGRESS_TITLE",L"Generating riddle..."));
-
-    minimumPoints->Set(L"4000");
+    generateTask.minimumPoints=4000;
 
     Observe(generateAction);
     Observe(solveAction);
@@ -165,15 +179,17 @@ public:
     gameArea->GetFinishedModel()->Off(); // Switch it off initialy and on after config reading!
     Observe(gameArea->GetFocusChangedAction());
     Observe(gameArea->GetFinishedModel());
+
+    Lum::Images::loader->LoadApplication(L"Statistics.png",statisticsImage);
+    Lum::Images::loader->LoadApplication(L"Solve.png",solveImage);
+    Lum::Images::loader->LoadApplication(L"Generate.png",generateImage);
   }
 
   void PreInit()
   {
-    Lum::Array                 *array;
-    Lum::Combo                 *combo;
-    Lum::Label                 *label;
-    Lum::Model::StringTableRef pointsModel;
-    Lum::Panel                 *horiz,*vert;
+    Lum::Array *array;
+    Lum::Label *label;
+    Lum::Panel *horiz,*vert;
 
     horiz=Lum::HPanel::Create(true,true);
 
@@ -186,21 +202,7 @@ public:
 
     vert=Lum::VPanel::Create(false,true);
 
-    label=Lum::Label::Create(false,true);
-
-    pointsModel=new Lum::Model::StringTable();
-    pointsModel->Append(L"2500");
-    pointsModel->Append(L"4000");
-    pointsModel->Append(L"8000");
-    pointsModel->Append(L"15000");
-    pointsModel->Append(L"25000");
-
-    combo=new Lum::TextEditCombo();
-    combo->SetFlex(true,false);
-    combo->SetModel(minimumPoints);
-    combo->SetTableModel(pointsModel);
-    dynamic_cast<Lum::TextEditCombo*>(combo)->GetString()->SetAlignment(Lum::String::right);
-    label->AddLabel(_(L"POINTS_MIN",L"Min. Points:"),combo);
+    label=Lum::Label::Create(true,false);
 
     rating=new Lum::Text();
     rating->SetFlex(true,false);
@@ -212,13 +214,13 @@ public:
 
     vert->Add(label);
 
-    vert->AddSpace(true);
-
     if (Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()) {
+      vert->AddSpace();
+
       array=new Lum::Array();
       array->SetFlex(true,true);
       array->SetHorizontalCount(3);
-      array->SetSpace(true,false);
+      array->SetSpace(false,false);
 
       for (size_t i=0; i<Sudoku::dimension; i++) {
         array->Add(Lum::Toggle::Create(Lum::Base::NumberToWString(i+1),keypad[i],true,true));
@@ -226,18 +228,39 @@ public:
 
       vert->Add(array);
 
+      vert->Add(Lum::Toggle::Create(L"Mark",mark,true,false));
+
+      vert->AddSpace();
+    }
+    else {
       vert->AddSpace(true);
     }
 
     array=new Lum::Array();
     array->SetFlex(true,false);
-    array->SetHorizontalCount(1);
+    array->SetHorizontalCount(3);
     array->SetSpace(false,false);
 
+    if (statisticsImage.Valid()) {
+      array->Add(Lum::Button::Create(Lum::Image::Create(statisticsImage),statAction,true,false));
+    }
+    else {
+      array->Add(Lum::Button::Create(_(L"GAME_STAT",L"S_tatistics"),statAction,true,false));
+    }
 
-    array->Add(Lum::Button::Create(_(L"GAME_STAT",L"S_tatistics"),statAction,true,false));
-    array->Add(Lum::Button::Create(_(L"SOLVE",L"_Solve"),solveAction,true,false));
-    array->Add(Lum::Button::Create(_(L"GENERATE",L"_Generate"),generateAction,true,false));
+    if (solveImage.Valid()) {
+      array->Add(Lum::Button::Create(Lum::Image::Create(solveImage),solveAction,true,false));
+    }
+    else {
+      array->Add(Lum::Button::Create(_(L"SOLVE",L"_Solve"),solveAction,true,false));
+    }
+
+    if (generateImage.Valid()) {
+      array->Add(Lum::Button::Create(Lum::Image::Create(generateImage),generateAction,true,false));
+    }
+    else {
+      array->Add(Lum::Button::Create(_(L"GENERATE",L"_New"),generateAction,true,false));
+    }
 
     vert->Add(array);
 
@@ -272,58 +295,111 @@ public:
     gameArea->GetFinishedModel()->Off();
     gameArea->SetModel(riddle,solution);
 
-    if (!Lum::Base::WStringToNumber(minimumPoints->Get(),generateTask.minimumPoints) ||
-        generateTask.minimumPoints<0) {
-      Lum::Dlg::Msg::ShowOk(this,
-                            _(L"INVALID_TITLE",L"Invalid value for 'minimum points'!"),
-                            _(L"INVALID_BODY",
-                              L"The value you have entered as minimum points for\n"
-                              L"riddle generation is not a valid number!"));
-      return;
-    }
-
     generateTask.Start();
   }
 
   void UpdateKeypad()
   {
-    for (size_t i=0; i<Sudoku::dimension; i++) {
-      if (gameArea->GetFinishedModel()->Get() || !gameArea->IsFocusedCellEditable()) {
-        keypad[i]->Set(false);
-        keypad[i]->Disable();
-      }
-      else {
-        keypad[i]->Enable();
-      }
-    }
-
-    if (gameArea->GetFinishedModel()->Get() || !gameArea->IsFocusedCellEditable()) {
+    if (lock) {
       return;
     }
 
-    size_t value=gameArea->GetFocusedCellValue();
+    lock=true;
+    if (gameArea->GetFinishedModel()->Get() || !gameArea->IsFocusedCellEditable()) {
+      for (size_t i=0; i<Sudoku::dimension; i++) {
+        keypad[i]->Set(false);
+        keypad[i]->Disable();
+      }
 
-    for (size_t i=0; i<Sudoku::dimension; i++) {
-      keypad[i]->Set(value & (1 << i));
+      lock=false;
+      return;
     }
+
+    if (gameArea->IsCellInMarkModus()) {
+      size_t value=gameArea->GetFocusedCellMarks();
+      size_t marks=0;
+
+      for (size_t i=0; i<Sudoku::dimension; i++) {
+        keypad[i]->Enable();
+        keypad[i]->Set(value & (1 << i));
+
+        if (value & (1 << i)) {
+          marks++;
+        }
+      }
+
+      if (marks==1) {
+        mark->Enable();
+        mark->Set(gameArea->IsCellInMarkModus());
+      }
+      else {
+        mark->Set(marks>1);
+        mark->Disable();
+      }
+    }
+    else {
+      size_t value=gameArea->GetFocusedCellValue();
+
+      for (size_t i=0; i<Sudoku::dimension; i++) {
+        keypad[i]->Enable();
+        keypad[i]->Set(i+1==value);
+      }
+
+      mark->Set(false);
+      mark->Enable();
+    }
+
+    lock=false;
   }
 
   void UpdateCell()
   {
+    if (lock) {
+      return;
+    }
+
     if (gameArea->GetFinishedModel()->Get() || !gameArea->IsFocusedCellEditable()) {
       // We ignore possible changes if there is no valid game
       return;
     }
 
-    size_t value=0;
+    lock=true;
 
+    size_t marks=0;
     for (size_t i=0; i<Sudoku::dimension; i++) {
       if (keypad[i].Valid() && keypad[i]->Get()) {
-        value|=(1 << i);
+        marks++;
       }
     }
 
-    gameArea->SetFocusedCellValue(value);
+    if (marks==1) {
+      mark->Enable();
+    }
+    else {
+      mark->Set(marks>1);
+      mark->Disable();
+    }
+
+    if (mark->Get()) {
+      size_t value=0;
+
+      for (size_t i=0; i<Sudoku::dimension; i++) {
+        if (keypad[i].Valid() && keypad[i]->Get()) {
+          value|=(1 << i);
+        }
+      }
+
+      gameArea->SetFocusedCellMarks(value);
+    }
+    else {
+      for (size_t i=0; i<Sudoku::dimension; i++) {
+        if (keypad[i].Valid() && keypad[i]->Get()) {
+          gameArea->SetFocusedCellValue(i+1);
+        }
+      }
+    }
+
+    lock=false;
   }
 
   void SetRiddle()
@@ -362,6 +438,7 @@ public:
 
       if (gameArea->GetFinishedModel()->Get()) {
         solveAction->Disable();
+        statAction->Disable();
       }
 
       gameArea->GetFinishedModel()->On(false);
@@ -374,19 +451,13 @@ public:
       SaveConfig();
     }
     else if (model==generateAction && generateAction->IsFinished()) {
-      if (Lum::Dlg::Msg::Ask(this,
-                             _(L"GENERATE_TITLE",L"Are you sure?"),
-                             _(L"GENERATE_BODY",
-                               L"Are you sure that you want to generate a new riddle?\n"
-                               L"\n"
-                               L"Generation cannot be undone! The old riddle\n"
-                               L"will be lost!"),
-                             _(L"GENERATE_BUTTONS",L"_Generate!*|_Cancel^"))==0) {
+      if (ShowGenerateDialog(this,generateTask.minimumPoints)) {
         GenerateRiddle();
       }
     }
     else if (model==generateTask.GetFinishedAction() && generateTask.GetFinishedAction()->IsFinished()) {
       solveAction->Enable();
+      statAction->Enable();
       SetRiddle();
       gameArea->GetFinishedModel()->On(false);
     }
@@ -405,6 +476,7 @@ public:
         gameArea->Solve();
         gameArea->GetFinishedModel()->On(false);
         solveAction->Disable();
+        statAction->Disable();
       }
     }
     else if (model==statAction && statAction->IsFinished()) {
@@ -447,11 +519,15 @@ public:
                               L"the given riddle!\n"
                               L"Congratulations!"));
       solveAction->Disable();
+      statAction->Disable();
       UpdateKeypad();
     }
     else if (model==gameArea->GetFocusChangedAction() && gameArea->GetFocusChangedAction()->IsFinished()) {
       UpdateKeypad();
     }
+    else if (model==mark) {
+      UpdateCell();
+    }
     else {
       for (size_t i=0; i<Sudoku::dimension; i++) {
         if (model==keypad[i]) {
@@ -481,6 +557,20 @@ public:
     info.SetCopyright(L"(c) 2006, Tim Teulings");
     info.SetLicense(L"GNU Public License");
 
+    Lum::Base::Path path;
+
+    path.SetNativeDir(Lum::Base::Path::GetApplicationDataDir());
+    path.AppendDir(L"icons");
+
+    Lum::Images::loader->AddApplicationSearchPath(path);
+
+    path.SetRoot(L"");
+    path.SetDir(L"data");
+    path.AppendDir(L"icons");
+    path.AppendDir(L"16x16");
+    Lum::Images::loader->AddApplicationSearchPath(path);
+
+
     return Lum::OS::MainDialog<MainDialog>::Prepare();
   }
 
